from setuptools import setup

setup(
    name='better-log',
    version='0.0.2',
    description='a better log',
    url='https://github.com/iammyz/better_log',
    author='mengyunzhuo',
    author_email='282053798@qq.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10'
    ],
    keywords='log python',
    packages=['better_log'],
)
